@@ -19,36 +19,55 @@ import threading
 import time
 import weakref
 
-from pymongo import thread_util
 from pymongo.monotonic import time as _time
 
 
 class PeriodicExecutor(object):
-    def __init__(self, condition_class, interval, min_interval, target):
+    def __init__(self, interval, min_interval, target, name=None):
         """"Run a target function periodically on a background thread.
 
         If the target's return value is false, the executor stops.
 
         :Parameters:
-          - `condition_class`: A class like threading.Condition.
           - `interval`: Seconds between calls to `target`.
           - `min_interval`: Minimum seconds between calls if `wake` is
             called very often.
           - `target`: A function.
+          - `name`: A name to give the underlying thread.
         """
-        self._event = thread_util.Event(condition_class)
+        # threading.Event and its internal condition variable are expensive
+        # in Python 2, see PYTHON-983. Use a boolean to know when to wake.
+        # The executor's design is constrained by several Python issues, see
+        # "periodic_executor.rst" in this repository.
+        self._event = False
         self._interval = interval
         self._min_interval = min_interval
         self._target = target
         self._stopped = False
         self._thread = None
+        self._name = name
+
+        self._thread_will_exit = False
+        self._lock = threading.Lock()
 
     def open(self):
         """Start. Multiple calls have no effect.
 
         Not safe to call from multiple threads at once.
         """
-        self._stopped = False
+        with self._lock:
+            if self._thread_will_exit:
+                # If the background thread has read self._stopped as True
+                # there is a chance that it has not yet exited. The call to
+                # join should not block indefinitely because there is no
+                # other work done outside the while loop in self._run.
+                try:
+                    self._thread.join()
+                except ReferenceError:
+                    # Thread terminated.
+                    pass
+            self._thread_will_exit = False
+            self._stopped = False
         started = False
         try:
             started = self._thread and self._thread.is_alive()
@@ -57,7 +76,7 @@ class PeriodicExecutor(object):
             pass
 
         if not started:
-            thread = threading.Thread(target=self._run)
+            thread = threading.Thread(target=self._run, name=self._name)
             thread.daemon = True
             self._thread = weakref.proxy(thread)
             _register_executor(self)
@@ -68,9 +87,6 @@ class PeriodicExecutor(object):
 
         The dummy parameter allows an executor's close method to be a weakref
         callback; see monitor.py.
-
-        Since this can be called from a weakref callback during garbage
-        collection it must take no locks! That means it cannot call wake().
         """
         self._stopped = True
 
@@ -78,16 +94,23 @@ class PeriodicExecutor(object):
         if self._thread is not None:
             try:
                 self._thread.join(timeout)
-            except ReferenceError:
-                # Thread already terminated.
+            except (ReferenceError, RuntimeError):
+                # Thread already terminated, or not yet started.
                 pass
 
     def wake(self):
         """Execute the target function soon."""
-        self._event.set()
+        self._event = True
+
+    def __should_stop(self):
+        with self._lock:
+            if self._stopped:
+                self._thread_will_exit = True
+                return True
+            return False
 
     def _run(self):
-        while not self._stopped:
+        while not self.__should_stop():
             try:
                 if not self._target():
                     self._stopped = True
@@ -98,17 +121,12 @@ class PeriodicExecutor(object):
 
             deadline = _time() + self._interval
 
-            # Avoid running too frequently if wake() is called very often.
-            time.sleep(self._min_interval)
-
-            # Until the deadline, wake often to check if close() was called.
             while not self._stopped and _time() < deadline:
-                # Our Event's wait returns True if set, else False.
-                if self._event.wait(0.1):
-                    # Someone called wake().
-                    break
+                time.sleep(self._min_interval)
+                if self._event:
+                    break  # Early wake.
 
-            self._event.clear()
+            self._event = False
 
 
 # _EXECUTORS has a weakref to each running PeriodicExecutor. Once started,
@@ -130,6 +148,9 @@ def _on_executor_deleted(ref):
 
 
 def _shutdown_executors():
+    if _EXECUTORS is None:
+        return
+
     # Copy the set. Stopping threads has the side effect of removing executors.
     executors = list(_EXECUTORS)
 
