@@ -30,13 +30,18 @@ from pymongo import MongoClient
 from pymongo.auth import HAVE_KERBEROS, _build_credentials_tuple
 from pymongo.errors import OperationFailure
 from pymongo.read_preferences import ReadPreference
-from test import client_context, host, port, SkipTest, unittest, Version
-from test.utils import delay
+from test import client_context, SkipTest, unittest, Version
+from test.utils import delay, rs_or_single_client_noauth, single_client_noauth
 
-# YOU MUST RUN KINIT BEFORE RUNNING GSSAPI TESTS.
+# YOU MUST RUN KINIT BEFORE RUNNING GSSAPI TESTS ON UNIX.
 GSSAPI_HOST = os.environ.get('GSSAPI_HOST')
 GSSAPI_PORT = int(os.environ.get('GSSAPI_PORT', '27017'))
-PRINCIPAL = os.environ.get('PRINCIPAL')
+GSSAPI_PRINCIPAL = os.environ.get('GSSAPI_PRINCIPAL')
+GSSAPI_SERVICE_NAME = os.environ.get('GSSAPI_SERVICE_NAME', 'mongodb')
+GSSAPI_CANONICALIZE = os.environ.get('GSSAPI_CANONICALIZE', 'false')
+GSSAPI_SERVICE_REALM = os.environ.get('GSSAPI_SERVICE_REALM')
+GSSAPI_PASS = os.environ.get('GSSAPI_PASS')
+GSSAPI_DB = os.environ.get('GSSAPI_DB', 'test')
 
 SASL_HOST = os.environ.get('SASL_HOST')
 SASL_PORT = int(os.environ.get('SASL_PORT', '27017'))
@@ -71,8 +76,18 @@ class TestGSSAPI(unittest.TestCase):
     def setUpClass(cls):
         if not HAVE_KERBEROS:
             raise SkipTest('Kerberos module not available.')
-        if not GSSAPI_HOST or not PRINCIPAL:
-            raise SkipTest('Must set GSSAPI_HOST and PRINCIPAL to test GSSAPI')
+        if not GSSAPI_HOST or not GSSAPI_PRINCIPAL:
+            raise SkipTest(
+               'Must set GSSAPI_HOST and GSSAPI_PRINCIPAL to test GSSAPI')
+        cls.service_realm_required = (
+            GSSAPI_SERVICE_REALM is not None and
+            GSSAPI_SERVICE_REALM not in GSSAPI_PRINCIPAL)
+        mech_properties = 'SERVICE_NAME:%s' % (GSSAPI_SERVICE_NAME,)
+        mech_properties += (
+            ',CANONICALIZE_HOST_NAME:%s' % (GSSAPI_CANONICALIZE,))
+        if GSSAPI_SERVICE_REALM is not None:
+            mech_properties += ',SERVICE_REALM:%s' % (GSSAPI_SERVICE_REALM,)
+        cls.mech_properties = mech_properties
 
     def test_credentials_hashing(self):
         # GSSAPI credentials are properly hashed.
@@ -95,74 +110,97 @@ class TestGSSAPI(unittest.TestCase):
         self.assertEqual(3, len(set([creds0, creds1, creds2, creds3])))
 
     def test_gssapi_simple(self):
-        # Call authenticate() without authMechanismProperties.
         client = MongoClient(GSSAPI_HOST, GSSAPI_PORT)
-        self.assertTrue(client.test.authenticate(PRINCIPAL,
-                                                 mechanism='GSSAPI'))
-        client.test.collection.find_one()
+        db = client[GSSAPI_DB]
+        if GSSAPI_PASS is not None:
+            uri = ('mongodb://%s:%s@%s:%d/?authMechanism='
+                   'GSSAPI' % (quote_plus(GSSAPI_PRINCIPAL),
+                               GSSAPI_PASS,
+                               GSSAPI_HOST,
+                               GSSAPI_PORT))
+        else:
+            uri = ('mongodb://%s@%s:%d/?authMechanism='
+                   'GSSAPI' % (quote_plus(GSSAPI_PRINCIPAL),
+                               GSSAPI_HOST,
+                               GSSAPI_PORT))
+
+        if not self.service_realm_required:
+            # Call authenticate() without authMechanismProperties.
+            self.assertTrue(db.authenticate(GSSAPI_PRINCIPAL,
+                                            GSSAPI_PASS,
+                                            mechanism='GSSAPI'))
+            db.collection.find_one()
+
+            # Log in using URI, without authMechanismProperties.
+            client = MongoClient(uri)
+            db = client[GSSAPI_DB]
+            db.collection.find_one()
 
-        # Log in using URI, without authMechanismProperties.
-        uri = ('mongodb://%s@%s:%d/?authMechanism='
-               'GSSAPI' % (quote_plus(PRINCIPAL), GSSAPI_HOST, GSSAPI_PORT))
-        client = MongoClient(uri)
-        client.test.collection.find_one()
 
         # Call authenticate() with authMechanismProperties.
-        self.assertTrue(client.test.authenticate(
-            PRINCIPAL, mechanism='GSSAPI',
-            authMechanismProperties='SERVICE_NAME:mongodb'))
-        client.test.collection.find_one()
+        self.assertTrue(db.authenticate(
+            GSSAPI_PRINCIPAL,
+            GSSAPI_PASS,
+            mechanism='GSSAPI',
+            authMechanismProperties=self.mech_properties))
+        db.collection.find_one()
 
         # Log in using URI, with authMechanismProperties.
-        uri = ('mongodb://%s@%s:%d/?authMechanism='
-               'GSSAPI;authMechanismProperties'
-               '=SERVICE_NAME:mongodb' % (quote_plus(PRINCIPAL),
-                                          GSSAPI_HOST, GSSAPI_PORT))
-        client = MongoClient(uri)
-        client.test.collection.find_one()
+        mech_uri = uri + '&authMechanismProperties=%s' % (self.mech_properties,)
+        client = MongoClient(mech_uri)
+        client[GSSAPI_DB].collection.find_one()
 
         set_name = client.admin.command('ismaster').get('setName')
         if set_name:
             client = MongoClient(GSSAPI_HOST,
-                                 port=GSSAPI_PORT,
+                                 GSSAPI_PORT,
                                  replicaSet=set_name)
-            # Without authMechanismProperties
-            self.assertTrue(client.test.authenticate(PRINCIPAL,
-                                                     mechanism='GSSAPI'))
-            client.database_names()
-            uri = ('mongodb://%s@%s:%d/?authMechanism=GSSAPI;replicaSet'
-                   '=%s' % (quote_plus(PRINCIPAL),
-                            GSSAPI_HOST, GSSAPI_PORT, str(set_name)))
-            client = MongoClient(uri)
-            client.database_names()
+            db = client[GSSAPI_DB]
+
+            if not self.service_realm_required:
+                # Without authMechanismProperties
+                self.assertTrue(db.authenticate(GSSAPI_PRINCIPAL,
+                                                GSSAPI_PASS,
+                                                mechanism='GSSAPI'))
+                db.collection_names()
+                uri = uri + '&replicaSet=%s' % (str(set_name),)
+                client = MongoClient(uri)
+                db = client[GSSAPI_DB]
+                db.collection_names()
 
             # With authMechanismProperties
-            self.assertTrue(client.test.authenticate(
-                PRINCIPAL, mechanism='GSSAPI',
-                authMechanismProperties='SERVICE_NAME:mongodb'))
-            client.database_names()
-            uri = ('mongodb://%s@%s:%d/?authMechanism=GSSAPI;replicaSet'
-                   '=%s;authMechanismProperties'
-                   '=SERVICE_NAME:mongodb' % (quote_plus(PRINCIPAL),
-                                              GSSAPI_HOST,
-                                              GSSAPI_PORT,
-                                              str(set_name)))
-            client = MongoClient(uri)
-            client.database_names()
+            self.assertTrue(db.authenticate(
+                GSSAPI_PRINCIPAL,
+                GSSAPI_PASS,
+                mechanism='GSSAPI',
+                authMechanismProperties=self.mech_properties))
+            db.collection_names()
+            mech_uri = mech_uri + '&replicaSet=%s' % (str(set_name),)
+            client = MongoClient(mech_uri)
+            client[GSSAPI_DB].collection_names()
 
     def test_gssapi_threaded(self):
 
-        client = MongoClient(GSSAPI_HOST)
-        self.assertTrue(client.test.authenticate(PRINCIPAL,
-                                                 mechanism='GSSAPI'))
+        client = MongoClient(GSSAPI_HOST, GSSAPI_PORT)
+        db = client[GSSAPI_DB]
+        self.assertTrue(
+            db.authenticate(GSSAPI_PRINCIPAL,
+                            GSSAPI_PASS,
+                            mechanism='GSSAPI',
+                            authMechanismProperties=self.mech_properties))
+
 
         # Need one document in the collection. AutoAuthenticateThread does
         # collection.find_one with a 1-second delay, forcing it to check out
         # multiple sockets from the pool concurrently, proving that
         # auto-authentication works with GSSAPI.
-        collection = client.test.collection
-        collection.drop()
-        collection.insert_one({'_id': 1})
+        collection = db.test
+        if collection.count() == 0:
+            try:
+                collection.drop()
+                collection.insert_one({'_id': 1})
+            except OperationFailure:
+                raise SkipTest("User must be able to write.")
 
         threads = []
         for _ in range(4):
@@ -176,11 +214,15 @@ class TestGSSAPI(unittest.TestCase):
         set_name = client.admin.command('ismaster').get('setName')
         if set_name:
             client = MongoClient(GSSAPI_HOST,
+                                 GSSAPI_PORT,
                                  replicaSet=set_name,
                                  readPreference='secondary')
-            self.assertTrue(client.test.authenticate(PRINCIPAL,
-                                                     mechanism='GSSAPI'))
-            self.assertTrue(client.test.command('dbstats'))
+            db = client[GSSAPI_DB]
+            self.assertTrue(
+                db.authenticate(GSSAPI_PRINCIPAL,
+                                GSSAPI_PASS,
+                                mechanism='GSSAPI',
+                                authMechanismProperties=self.mech_properties))
 
             threads = []
             for _ in range(4):
@@ -263,14 +305,11 @@ class TestSASLPlain(unittest.TestCase):
         self.assertRaises(OperationFailure, bad_pwd.admin.command, 'ismaster')
 
 
-
 class TestSCRAMSHA1(unittest.TestCase):
 
     @client_context.require_auth
     @client_context.require_version_min(2, 7, 2)
     def setUp(self):
-        self.replica_set_name = client_context.replica_set_name
-
         # Before 2.7.7, SCRAM-SHA-1 had to be enabled from the command line.
         if client_context.version < Version(2, 7, 7):
             cmd_line = client_context.cmd_line
@@ -279,49 +318,44 @@ class TestSCRAMSHA1(unittest.TestCase):
                     {}).get('authenticationMechanisms', ''):
                 raise SkipTest('SCRAM-SHA-1 mechanism not enabled')
 
-        client = client_context.rs_or_standalone_client
+        client = client_context.client
         client.pymongo_test.add_user(
             'user', 'pass',
             roles=['userAdmin', 'readWrite'],
             writeConcern={'w': client_context.w})
 
     def test_scram_sha1(self):
-        client = MongoClient(host, port)
+        host, port = client_context.host, client_context.port
+        client = rs_or_single_client_noauth()
         self.assertTrue(client.pymongo_test.authenticate(
             'user', 'pass', mechanism='SCRAM-SHA-1'))
         client.pymongo_test.command('dbstats')
 
-        client = MongoClient('mongodb://user:pass@%s:%d/pymongo_test'
-                             '?authMechanism=SCRAM-SHA-1' % (host, port))
+        client = rs_or_single_client_noauth(
+            'mongodb://user:pass@%s:%d/pymongo_test?authMechanism=SCRAM-SHA-1'
+            % (host, port))
         client.pymongo_test.command('dbstats')
 
-        if self.replica_set_name:
-            client = MongoClient(host, port,
-                                 replicaSet='%s' % (self.replica_set_name,))
-            self.assertTrue(client.pymongo_test.authenticate(
-                'user', 'pass', mechanism='SCRAM-SHA-1'))
-            client.pymongo_test.command('dbstats')
-
+        if client_context.is_rs:
             uri = ('mongodb://user:pass'
                    '@%s:%d/pymongo_test?authMechanism=SCRAM-SHA-1'
-                   '&replicaSet=%s' % (host, port, self.replica_set_name))
-            client = MongoClient(uri)
+                   '&replicaSet=%s' % (host, port,
+                                       client_context.replica_set_name))
+            client = single_client_noauth(uri)
             client.pymongo_test.command('dbstats')
             db = client.get_database(
                 'pymongo_test', read_preference=ReadPreference.SECONDARY)
             db.command('dbstats')
 
     def tearDown(self):
-        client_context.rs_or_standalone_client.pymongo_test.remove_user('user')
+        client_context.client.pymongo_test.remove_user('user')
 
 
 class TestAuthURIOptions(unittest.TestCase):
 
     @client_context.require_auth
     def setUp(self):
-        client = MongoClient(host, port)
-        response = client.admin.command('ismaster')
-        self.replica_set_name = str(response.get('setName', ''))
+        client = rs_or_single_client_noauth()
         client_context.client.admin.add_user('admin', 'pass',
                                              roles=['userAdminAnyDatabase',
                                                     'dbAdminAnyDatabase',
@@ -331,12 +365,10 @@ class TestAuthURIOptions(unittest.TestCase):
         client.pymongo_test.add_user('user', 'pass',
                                      roles=['userAdmin', 'readWrite'])
 
-        if self.replica_set_name:
-            # GLE requires authentication.
-            client.admin.authenticate('admin', 'pass')
+        if client_context.is_rs:
             # Make sure the admin user is replicated after calling add_user
             # above. This avoids a race in the replica set tests below.
-            client.admin.command('getLastError', w=len(response['hosts']))
+            client.admin.command('getLastError', w=client_context.w)
         self.client = client
 
     def tearDown(self):
@@ -349,13 +381,15 @@ class TestAuthURIOptions(unittest.TestCase):
 
     def test_uri_options(self):
         # Test default to admin
-        client = MongoClient('mongodb://admin:pass@%s:%d' % (host, port))
+        host, port = client_context.host, client_context.port
+        client = rs_or_single_client_noauth(
+            'mongodb://admin:pass@%s:%d' % (host, port))
         self.assertTrue(client.admin.command('dbstats'))
 
-        if self.replica_set_name:
-            uri = ('mongodb://admin:pass'
-                   '@%s:%d/?replicaSet=%s' % (host, port, self.replica_set_name))
-            client = MongoClient(uri)
+        if client_context.is_rs:
+            uri = ('mongodb://admin:pass@%s:%d/?replicaSet=%s' % (
+                host, port, client_context.replica_set_name))
+            client = single_client_noauth(uri)
             self.assertTrue(client.admin.command('dbstats'))
             db = client.get_database(
                 'admin', read_preference=ReadPreference.SECONDARY)
@@ -363,14 +397,14 @@ class TestAuthURIOptions(unittest.TestCase):
 
         # Test explicit database
         uri = 'mongodb://user:pass@%s:%d/pymongo_test' % (host, port)
-        client = MongoClient(uri)
+        client = rs_or_single_client_noauth(uri)
         self.assertRaises(OperationFailure, client.admin.command, 'dbstats')
         self.assertTrue(client.pymongo_test.command('dbstats'))
 
-        if self.replica_set_name:
-            uri = ('mongodb://user:pass@%s:%d'
-                   '/pymongo_test?replicaSet=%s' % (host, port, self.replica_set_name))
-            client = MongoClient(uri)
+        if client_context.is_rs:
+            uri = ('mongodb://user:pass@%s:%d/pymongo_test?replicaSet=%s' % (
+                host, port, client_context.replica_set_name))
+            client = single_client_noauth(uri)
             self.assertRaises(OperationFailure,
                               client.admin.command, 'dbstats')
             self.assertTrue(client.pymongo_test.command('dbstats'))
@@ -381,15 +415,16 @@ class TestAuthURIOptions(unittest.TestCase):
         # Test authSource
         uri = ('mongodb://user:pass@%s:%d'
                '/pymongo_test2?authSource=pymongo_test' % (host, port))
-        client = MongoClient(uri)
+        client = rs_or_single_client_noauth(uri)
         self.assertRaises(OperationFailure,
                           client.pymongo_test2.command, 'dbstats')
         self.assertTrue(client.pymongo_test.command('dbstats'))
 
-        if self.replica_set_name:
+        if client_context.is_rs:
             uri = ('mongodb://user:pass@%s:%d/pymongo_test2?replicaSet='
-                   '%s;authSource=pymongo_test' % (host, port, self.replica_set_name))
-            client = MongoClient(uri)
+                   '%s;authSource=pymongo_test' % (
+                host, port, client_context.replica_set_name))
+            client = single_client_noauth(uri)
             self.assertRaises(OperationFailure,
                               client.pymongo_test2.command, 'dbstats')
             self.assertTrue(client.pymongo_test.command('dbstats'))
@@ -404,7 +439,7 @@ class TestDelegatedAuth(unittest.TestCase):
     @client_context.require_version_max(2, 5, 3)
     @client_context.require_version_min(2, 4, 0)
     def setUp(self):
-        self.client = client_context.rs_or_standalone_client
+        self.client = client_context.client
 
     def tearDown(self):
         self.client.pymongo_test.remove_user('user')
@@ -420,7 +455,7 @@ class TestDelegatedAuth(unittest.TestCase):
         self.client.pymongo_test2.add_user('user',
                                            userSource='pymongo_test',
                                            roles=['read'])
-        auth_c = MongoClient(host, port)
+        auth_c = rs_or_single_client_noauth()
         self.assertRaises(OperationFailure,
                           auth_c.pymongo_test2.foo.find_one)
         # Auth must occur on the db where the user is defined.
