@@ -21,7 +21,6 @@ sys.path[0:0] = [""]
 from bson import InvalidDocument, SON
 from bson.objectid import ObjectId
 from bson.py3compat import string_type
-from pymongo import MongoClient
 from pymongo.operations import *
 from pymongo.common import partition_node
 from pymongo.errors import (BulkWriteError,
@@ -31,11 +30,13 @@ from pymongo.errors import (BulkWriteError,
 from pymongo.write_concern import WriteConcern
 from test import (client_context,
                   unittest,
-                  host,
-                  port,
                   IntegrationTest,
                   SkipTest)
-from test.utils import oid_generated_on_client, remove_all_users, wait_until
+from test.utils import (oid_generated_on_client,
+                        remove_all_users,
+                        rs_or_single_client_noauth,
+                        single_client,
+                        wait_until)
 
 
 class BulkTestBase(IntegrationTest):
@@ -138,6 +139,41 @@ class TestBulk(BulkTestBase):
         # No error.
         bulk.find({})
 
+    @client_context.require_version_min(3, 1, 9, -1)
+    def test_bypass_document_validation_bulk_op(self):
+
+        # Test insert
+        self.coll.insert_one({"z": 0})
+        self.db.command(SON([("collMod", "test"),
+                             ("validator", {"z": {"$gte": 0}})]))
+        bulk = self.coll.initialize_ordered_bulk_op(
+            bypass_document_validation=False)
+        bulk.insert({"z": -1}) # error
+        self.assertRaises(BulkWriteError, bulk.execute)
+        self.assertEqual(0, self.coll.count({"z": -1}))
+
+        bulk = self.coll.initialize_ordered_bulk_op(
+            bypass_document_validation=True)
+        bulk.insert({"z": -1})
+        bulk.execute()
+        self.assertEqual(1, self.coll.count({"z": -1}))
+
+        self.coll.insert_one({"z": 0})
+        self.db.command(SON([("collMod", "test"),
+                             ("validator", {"z": {"$gte": 0}})]))
+        bulk = self.coll.initialize_unordered_bulk_op(
+            bypass_document_validation=False)
+        bulk.insert({"z": -1}) # error
+        self.assertRaises(BulkWriteError, bulk.execute)
+        self.assertEqual(1, self.coll.count({"z": -1}))
+
+        bulk = self.coll.initialize_unordered_bulk_op(
+            bypass_document_validation=True)
+        bulk.insert({"z": -1})
+        bulk.execute()
+        self.assertEqual(2, self.coll.count({"z": -1}))
+        self.coll.drop()
+
     def test_insert(self):
         expected = {
             'nMatched': 0,
@@ -937,7 +973,7 @@ class TestBulkWriteConcern(BulkTestBase):
         if cls.w > 1:
             for member in client_context.ismaster['hosts']:
                 if member != client_context.ismaster['primary']:
-                    cls.secondary = MongoClient(*partition_node(member))
+                    cls.secondary = single_client(*partition_node(member))
                     break
 
         # We tested wtimeout errors by specifying a write concern greater than
@@ -1209,7 +1245,7 @@ class TestBulkAuthorization(BulkTestBase):
     def test_readonly(self):
         # We test that an authorization failure aborts the batch and is raised
         # as OperationFailure.
-        cli = MongoClient(host, port)
+        cli = rs_or_single_client_noauth()
         db = cli.pymongo_test
         coll = db.test
         db.authenticate('readonly', 'pw')
@@ -1220,7 +1256,7 @@ class TestBulkAuthorization(BulkTestBase):
     def test_no_remove(self):
         # We test that an authorization failure aborts the batch and is raised
         # as OperationFailure.
-        cli = MongoClient(host, port)
+        cli = rs_or_single_client_noauth()
         db = cli.pymongo_test
         coll = db.test
         db.authenticate('noremove', 'pw')
