@@ -22,29 +22,30 @@ import struct
 import sys
 import time
 import traceback
-import warnings
 
 sys.path[0:0] = [""]
 
 from bson import BSON
 from bson.codec_options import CodecOptions
-from bson.py3compat import thread, u
+from bson.py3compat import thread
 from bson.son import SON
 from bson.tz_util import utc
 from pymongo import auth, message
+from pymongo.common import _UUID_REPRESENTATIONS
 from pymongo.cursor import CursorType
 from pymongo.database import Database
 from pymongo.errors import (AutoReconnect,
                             ConfigurationError,
                             ConnectionFailure,
                             InvalidName,
-                            OperationFailure,
-                            CursorNotFound,
+                            InvalidURI,
                             NetworkTimeout,
-                            InvalidURI)
-from pymongo.message import _CursorAddress
+                            OperationFailure,
+                            WriteConcernError)
+from pymongo.monitoring import (ServerHeartbeatListener,
+                                ServerHeartbeatStartedEvent)
 from pymongo.mongo_client import MongoClient
-from pymongo.pool import SocketInfo
+from pymongo.pool import SocketInfo, _METADATA
 from pymongo.read_preferences import ReadPreference
 from pymongo.server_selectors import (any_server_selector,
                                       writable_server_selector)
@@ -52,9 +53,6 @@ from pymongo.server_type import SERVER_TYPE
 from pymongo.write_concern import WriteConcern
 from test import (client_context,
                   client_knobs,
-                  host,
-                  pair,
-                  port,
                   SkipTest,
                   unittest,
                   IntegrationTest,
@@ -70,8 +68,10 @@ from test.utils import (assertRaisesExactly,
                         one,
                         connected,
                         wait_until,
+                        rs_client,
                         rs_or_single_client,
                         rs_or_single_client_noauth,
+                        single_client,
                         lazy_client_trial,
                         NTHREADS)
 
@@ -80,9 +80,10 @@ class ClientUnitTest(unittest.TestCase):
     """MongoClient tests that don't require a server."""
 
     @classmethod
+    @client_context.require_connection
     def setUpClass(cls):
-        cls.client = MongoClient(host, port, connect=False,
-                                 serverSelectionTimeoutMS=100)
+        cls.client = rs_or_single_client(connect=False,
+                                         serverSelectionTimeoutMS=100)
 
     def test_keyword_arg_defaults(self):
         client = MongoClient(socketTimeoutMS=None,
@@ -170,41 +171,199 @@ class ClientUnitTest(unittest.TestCase):
         self.assertRaises(TypeError, iterate)
 
     def test_get_default_database(self):
-        c = MongoClient("mongodb://%s:%d/foo" % (host, port), connect=False)
+        c = rs_or_single_client("mongodb://%s:%d/foo" % (client_context.host,
+                                                         client_context.port),
+                                connect=False)
         self.assertEqual(Database(c, 'foo'), c.get_default_database())
 
     def test_get_default_database_error(self):
         # URI with no database.
-        c = MongoClient("mongodb://%s:%d/" % (host, port), connect=False)
+        c = rs_or_single_client("mongodb://%s:%d/" % (client_context.host,
+                                                      client_context.port),
+                                connect=False)
         self.assertRaises(ConfigurationError, c.get_default_database)
 
     def test_get_default_database_with_authsource(self):
         # Ensure we distinguish database name from authSource.
-        uri = "mongodb://%s:%d/foo?authSource=src" % (host, port)
-        c = MongoClient(uri, connect=False)
+        uri = "mongodb://%s:%d/foo?authSource=src" % (
+            client_context.host, client_context.port)
+        c = rs_or_single_client(uri, connect=False)
         self.assertEqual(Database(c, 'foo'), c.get_default_database())
 
+    def test_primary_read_pref_with_tags(self):
+        # No tags allowed with "primary".
+        with self.assertRaises(ConfigurationError):
+            MongoClient('mongodb://host/?readpreferencetags=dc:east')
+
+        with self.assertRaises(ConfigurationError):
+            MongoClient('mongodb://host/?'
+                        'readpreference=primary&readpreferencetags=dc:east')
+
+    def test_metadata(self):
+        metadata = _METADATA.copy()
+        metadata['application'] = {'name': 'foobar'}
+        client = MongoClient(
+            "mongodb://foo:27017/?appname=foobar&connect=false")
+        options = client._MongoClient__options
+        self.assertEqual(options.pool_options.metadata, metadata)
+        client = MongoClient('foo', 27017, appname='foobar', connect=False)
+        options = client._MongoClient__options
+        self.assertEqual(options.pool_options.metadata, metadata)
+        # No error
+        MongoClient(appname='x' * 128)
+        self.assertRaises(ValueError, MongoClient, appname='x' * 129)
+
+    def test_kwargs_codec_options(self):
+        # Ensure codec options are passed in correctly
+        document_class = SON
+        tz_aware = True
+        uuid_representation_label = 'javaLegacy'
+        unicode_decode_error_handler = 'ignore'
+        tzinfo = utc
+        c = MongoClient(
+            document_class=document_class,
+            tz_aware=tz_aware,
+            uuidrepresentation=uuid_representation_label,
+            unicode_decode_error_handler=unicode_decode_error_handler,
+            tzinfo=tzinfo,
+            connect=False
+        )
+
+        self.assertEqual(c.codec_options.document_class, document_class)
+        self.assertEqual(c.codec_options.tz_aware, tz_aware)
+        self.assertEqual(
+            c.codec_options.uuid_representation,
+            _UUID_REPRESENTATIONS[uuid_representation_label])
+        self.assertEqual(
+            c.codec_options.unicode_decode_error_handler,
+            unicode_decode_error_handler)
+        self.assertEqual(c.codec_options.tzinfo, tzinfo)
+
+    def test_uri_codec_options(self):
+        # Ensure codec options are passed in correctly
+        uuid_representation_label = 'javaLegacy'
+        unicode_decode_error_handler = 'ignore'
+        uri = ("mongodb://%s:%d/foo?tz_aware=true&uuidrepresentation="
+               "%s&unicode_decode_error_handler=%s" % (
+                   client_context.host,
+                   client_context.port,
+                   uuid_representation_label,
+                   unicode_decode_error_handler))
+        c = MongoClient(uri, connect=False)
+
+        self.assertEqual(c.codec_options.tz_aware, True)
+        self.assertEqual(
+            c.codec_options.uuid_representation,
+            _UUID_REPRESENTATIONS[uuid_representation_label])
+        self.assertEqual(
+            c.codec_options.unicode_decode_error_handler,
+            unicode_decode_error_handler)
+
 
 class TestClient(IntegrationTest):
 
+    def test_max_idle_time_reaper(self):
+        with client_knobs(kill_cursor_frequency=0.1):
+            # Assert reaper doesn't remove sockets when maxIdleTimeMS not set
+            client = rs_or_single_client()
+            server = client._get_topology().select_server(any_server_selector)
+            with server._pool.get_socket({}) as sock_info:
+                pass
+            self.assertEqual(1, len(server._pool.sockets))
+            self.assertTrue(sock_info in server._pool.sockets)
+
+            # Assert reaper removes idle socket and replaces it with a new one
+            client = rs_or_single_client(maxIdleTimeMS=.5,
+                                         minPoolSize=1)
+            server = client._get_topology().select_server(any_server_selector)
+            with server._pool.get_socket({}) as sock_info:
+                pass
+            self.assertEqual(1, len(server._pool.sockets))
+            wait_until(lambda: sock_info not in server._pool.sockets,
+                       "reaper removes stale socket eventually")
+            wait_until(lambda: 1 == len(server._pool.sockets),
+                       "reaper replaces stale socket with new one")
+
+            # Assert reaper has removed idle socket and NOT replaced it
+            client = rs_or_single_client(maxIdleTimeMS=.5)
+            server = client._get_topology().select_server(any_server_selector)
+            with server._pool.get_socket({}):
+                pass
+            wait_until(
+                lambda: 0 == len(server._pool.sockets),
+                "stale socket reaped and new one NOT added to the pool")
+
+    def test_min_pool_size(self):
+        with client_knobs(kill_cursor_frequency=.1):
+            client = rs_or_single_client()
+            server = client._get_topology().select_server(any_server_selector)
+            self.assertEqual(0, len(server._pool.sockets))
+
+            # Assert that pool started up at minPoolSize
+            client = rs_or_single_client(minPoolSize=10)
+            server = client._get_topology().select_server(any_server_selector)
+            wait_until(lambda: 10 == len(server._pool.sockets),
+                       "pool initialized with 10 sockets")
+
+            # Assert that if a socket is closed, a new one takes its place
+            with server._pool.get_socket({}) as sock_info:
+                sock_info.close()
+            wait_until(lambda: 10 == len(server._pool.sockets),
+                       "a closed socket gets replaced from the pool")
+            self.assertFalse(sock_info in server._pool.sockets)
+
+    def test_max_idle_time_checkout(self):
+        # Use high frequency to test _get_socket_no_auth.
+        with client_knobs(kill_cursor_frequency=99999999):
+            client = rs_or_single_client(maxIdleTimeMS=.5)
+            server = client._get_topology().select_server(any_server_selector)
+            with server._pool.get_socket({}) as sock_info:
+                pass
+            self.assertEqual(1, len(server._pool.sockets))
+            time.sleep(1) #  Sleep so that the socket becomes stale.
+
+            with server._pool.get_socket({}) as new_sock_info:
+                self.assertNotEqual(sock_info, new_sock_info)
+            self.assertEqual(1, len(server._pool.sockets))
+            self.assertFalse(sock_info in server._pool.sockets)
+            self.assertTrue(new_sock_info in server._pool.sockets)
+
+            # Test that sockets are reused if maxIdleTimeMS is not set.
+            client = rs_or_single_client()
+            server = client._get_topology().select_server(any_server_selector)
+            with server._pool.get_socket({}) as sock_info:
+                pass
+            self.assertEqual(1, len(server._pool.sockets))
+            time.sleep(1)
+            with server._pool.get_socket({}) as new_sock_info:
+                self.assertEqual(sock_info, new_sock_info)
+            self.assertEqual(1, len(server._pool.sockets))
+
     def test_constants(self):
+        """This test uses MongoClient explicitly to make sure that host and
+        port are not overloaded.
+        """
+        host, port = client_context.host, client_context.port
         # Set bad defaults.
         MongoClient.HOST = "somedomainthatdoesntexist.org"
         MongoClient.PORT = 123456789
         with self.assertRaises(AutoReconnect):
-            connected(MongoClient(serverSelectionTimeoutMS=10))
+            connected(MongoClient(serverSelectionTimeoutMS=10,
+                                  **client_context.ssl_client_options))
 
         # Override the defaults. No error.
-        connected(MongoClient(host, port))
+        connected(MongoClient(host, port,
+                              **client_context.ssl_client_options))
 
         # Set good defaults.
         MongoClient.HOST = host
         MongoClient.PORT = port
 
         # No error.
-        connected(MongoClient())
+        connected(MongoClient(**client_context.ssl_client_options))
 
     def test_init_disconnected(self):
+        host, port = client_context.host, client_context.port
         c = rs_or_single_client(connect=False)
         # is_primary causes client to block until connected
         self.assertIsInstance(c.is_primary, bool)
@@ -243,14 +402,15 @@ class TestClient(IntegrationTest):
 
     def test_equality(self):
         c = connected(rs_or_single_client())
-        self.assertEqual(client_context.rs_or_standalone_client, c)
+        self.assertEqual(client_context.client, c)
 
         # Explicitly test inequality
-        self.assertFalse(client_context.rs_or_standalone_client != c)
+        self.assertFalse(client_context.client != c)
 
     def test_host_w_port(self):
         with self.assertRaises(ValueError):
-            connected(MongoClient("%s:1234567" % host, connectTimeoutMS=1,
+            connected(MongoClient("%s:1234567" % (client_context.host,),
+                                  connectTimeoutMS=1,
                                   serverSelectionTimeoutMS=10))
 
     def test_repr(self):
@@ -259,7 +419,7 @@ class TestClient(IntegrationTest):
 
         client = MongoClient(
             'mongodb://localhost:27017,localhost:27018/?replicaSet=replset'
-            '&connectTimeoutMS=12345',
+            '&connectTimeoutMS=12345&w=1&wtimeoutms=100',
             connect=False, document_class=SON)
 
         the_repr = repr(client)
@@ -269,23 +429,42 @@ class TestClient(IntegrationTest):
             "tz_aware=False, "
             "connect=False, ",
             the_repr)
-        self.assertIn("connecttimeoutms='12345'", the_repr)
-        self.assertIn("replicaset=", the_repr)
+        self.assertIn("connecttimeoutms=12345", the_repr)
+        self.assertIn("replicaset='replset'", the_repr)
+        self.assertIn("w=1", the_repr)
+        self.assertIn("wtimeoutms=100", the_repr)
 
         self.assertEqual(eval(the_repr), client)
 
-    @client_context.require_replica_set
-    def test_repr_replica_set(self):
-        self.assertIn("MongoClient(host=[", repr(self.client))
-        self.assertIn(pair, repr(self.client))
+        client = MongoClient("localhost:27017,localhost:27018",
+                             replicaSet='replset',
+                             connectTimeoutMS=12345,
+                             socketTimeoutMS=None,
+                             w=1,
+                             wtimeoutms=100,
+                             connect=False)
+        the_repr = repr(client)
+        self.assertIn('MongoClient(host=', the_repr)
+        self.assertIn(
+            "document_class=dict, "
+            "tz_aware=False, "
+            "connect=False, ",
+            the_repr)
+        self.assertIn("connecttimeoutms=12345", the_repr)
+        self.assertIn("replicaset='replset'", the_repr)
+        self.assertIn("sockettimeoutms=None", the_repr)
+        self.assertIn("w=1", the_repr)
+        self.assertIn("wtimeoutms=100", the_repr)
+
+        self.assertEqual(eval(the_repr), client)
 
     def test_getters(self):
-        self.assertEqual(client_context.client.address, (host, port))
-        self.assertEqual(client_context.nodes, self.client.nodes)
+        wait_until(lambda: client_context.nodes == self.client.nodes,
+                   "find all nodes")
 
     def test_database_names(self):
-        self.client.pymongo_test.test.insert_one({"dummy": u("object")})
-        self.client.pymongo_test_mike.test.insert_one({"dummy": u("object")})
+        self.client.pymongo_test.test.insert_one({"dummy": u"object"})
+        self.client.pymongo_test_mike.test.insert_one({"dummy": u"object"})
 
         dbs = self.client.database_names()
         self.assertTrue("pymongo_test" in dbs)
@@ -295,12 +474,18 @@ class TestClient(IntegrationTest):
         self.assertRaises(TypeError, self.client.drop_database, 5)
         self.assertRaises(TypeError, self.client.drop_database, None)
 
-        self.client.pymongo_test.test.insert_one({"dummy": u("object")})
-        self.client.pymongo_test2.test.insert_one({"dummy": u("object")})
+        self.client.pymongo_test.test.insert_one({"dummy": u"object"})
+        self.client.pymongo_test2.test.insert_one({"dummy": u"object"})
         dbs = self.client.database_names()
         self.assertIn("pymongo_test", dbs)
         self.assertIn("pymongo_test2", dbs)
         self.client.drop_database("pymongo_test")
+
+        if client_context.version.at_least(3, 3, 9) and client_context.is_rs:
+            wc_client = rs_or_single_client(w=len(client_context.nodes) + 1)
+            with self.assertRaises(WriteConcernError):
+                wc_client.drop_database('pymongo_test2')
+
         self.client.drop_database(self.client.pymongo_test2)
 
         raise SkipTest("This test often fails due to SERVER-2329")
@@ -328,6 +513,7 @@ class TestClient(IntegrationTest):
 
     @client_context.require_auth
     def test_auth_from_uri(self):
+        host, port = client_context.host, client_context.port
         self.client.admin.add_user("admin", "pass", roles=["root"])
         self.addCleanup(self.client.admin.remove_user, 'admin')
         self.addCleanup(remove_all_users, self.client.pymongo_test)
@@ -353,12 +539,12 @@ class TestClient(IntegrationTest):
             "mongodb://user:pass@%s:%d/pymongo_test" % (host, port)))
 
         # Auth with lazy connection.
-        rs_or_single_client(
+        rs_or_single_client_noauth(
             "mongodb://user:pass@%s:%d/pymongo_test" % (host, port),
             connect=False).pymongo_test.test.find_one()
 
         # Wrong password.
-        bad_client = rs_or_single_client(
+        bad_client = rs_or_single_client_noauth(
             "mongodb://user:wrong@%s:%d/pymongo_test" % (host, port),
             connect=False)
 
@@ -372,7 +558,8 @@ class TestClient(IntegrationTest):
         self.addCleanup(remove_all_users, self.client.pymongo_test)
 
         client = rs_or_single_client_noauth(
-            "mongodb://user1:pass@%s:%d/pymongo_test" % (host, port))
+            "mongodb://user1:pass@%s:%d/pymongo_test" % (
+                client_context.host, client_context.port))
 
         client.pymongo_test.test.find_one()
         with self.assertRaises(OperationFailure):
@@ -394,8 +581,9 @@ class TestClient(IntegrationTest):
 
     @client_context.require_auth
     def test_lazy_auth_raises_operation_failure(self):
-        lazy_client = rs_or_single_client(
-            "mongodb://user:wrong@%s/pymongo_test" % host, connect=False)
+        lazy_client = rs_or_single_client_noauth(
+            "mongodb://user:wrong@%s/pymongo_test" % (client_context.host,),
+            connect=False)
 
         assertRaisesExactly(
             OperationFailure, lazy_client.test.collection.find_one)
@@ -404,8 +592,9 @@ class TestClient(IntegrationTest):
         if not hasattr(socket, "AF_UNIX"):
             raise SkipTest("UNIX-sockets are not supported on this system")
 
-        mongodb_socket = '/tmp/mongodb-27017.sock'
-        encoded_socket = '%2Ftmp%2Fmongodb-27017.sock'
+        mongodb_socket = '/tmp/mongodb-%d.sock' % (client_context.port,)
+        encoded_socket = (
+            '%2Ftmp%2F' + 'mongodb-%d.sock' % (client_context.port,))
         if not os.access(mongodb_socket, os.R_OK):
             raise SkipTest("Socket file is not accessible")
 
@@ -415,64 +604,19 @@ class TestClient(IntegrationTest):
             uri = "mongodb://%s" % encoded_socket
 
         # Confirm we can do operations via the socket.
-        client = MongoClient(uri)
+        client = rs_or_single_client(uri)
         client.pymongo_test.test.insert_one({"dummy": "object"})
         dbs = client.database_names()
         self.assertTrue("pymongo_test" in dbs)
 
+        self.assertTrue(mongodb_socket in repr(client))
+
         # Confirm it fails with a missing socket.
         self.assertRaises(
             ConnectionFailure,
             connected, MongoClient("mongodb://%2Ftmp%2Fnon-existent.sock",
                                    serverSelectionTimeoutMS=100))
 
-    def test_fork(self):
-        # Test using a client before and after a fork.
-        if sys.platform == "win32":
-            raise SkipTest("Can't fork on windows")
-        try:
-            import multiprocessing
-        except ImportError:
-            raise SkipTest("No multiprocessing module")
-
-        db = self.client.pymongo_test
-
-        # Ensure a socket is opened before the fork.
-        db.test.find_one()
-
-        def f(pipe):
-            try:
-                kill_cursors_executor = self.client._kill_cursors_executor
-                servers = self.client._topology.select_servers(
-                    any_server_selector)
-
-                # In child, only the thread that called fork() is alive.
-                # The first operation should revive the rest.
-                db.test.find_one()
-                wait_until(
-                    lambda: all(s._monitor._executor._thread.is_alive()
-                                for s in servers),
-                    "restart monitor threads")
-
-                wait_until(lambda: kill_cursors_executor._thread.is_alive(),
-                           "restart kill-cursors executor")
-            except:
-                traceback.print_exc()  # Aid debugging.
-                pipe.send(True)
-
-        parent_pipe, child_pipe = multiprocessing.Pipe()
-        p = multiprocessing.Process(target=f, args=(child_pipe,))
-        p.start()
-        p.join(10)
-        child_pipe.close()
-
-        # Pipe will only have data if the child process failed.
-        try:
-            parent_pipe.recv()
-            self.fail()
-        except EOFError:
-            pass
-
     def test_document_class(self):
         c = self.client
         db = c.pymongo_test
@@ -598,13 +742,13 @@ class TestClient(IntegrationTest):
         else:
             auth_str = ""
 
-        uri = "mongodb://%s[::1]:%d" % (auth_str, port)
+        uri = "mongodb://%s[::1]:%d" % (auth_str, client_context.port)
         if client_context.is_rs:
             uri += '/?replicaSet=' + client_context.replica_set_name
 
         client = rs_or_single_client_noauth(uri)
-        client.pymongo_test.test.insert_one({"dummy": u("object")})
-        client.pymongo_test_bernie.test.insert_one({"dummy": u("object")})
+        client.pymongo_test.test.insert_one({"dummy": u"object"})
+        client.pymongo_test_bernie.test.insert_one({"dummy": u"object"})
 
         dbs = client.database_names()
         self.assertTrue("pymongo_test" in dbs)
@@ -712,85 +856,6 @@ class TestClient(IntegrationTest):
         new_sock_info = next(iter(pool.sockets))
         self.assertEqual(old_sock_info, new_sock_info)
 
-    def test_kill_cursors_with_cursoraddress(self):
-        if (client_context.is_mongos
-                and not client_context.version.at_least(2, 4, 7)):
-            # Old mongos sends incorrectly formatted error response when
-            # cursor isn't found, see SERVER-9738.
-            raise SkipTest("Can't test kill_cursors against old mongos")
-
-        self.collection = self.client.pymongo_test.test
-        self.collection.drop()
-
-        self.collection.insert_many([{'_id': i} for i in range(200)])
-        cursor = self.collection.find().batch_size(1)
-        next(cursor)
-        self.client.kill_cursors(
-            [cursor.cursor_id],
-            _CursorAddress(self.client.address, self.collection.full_name))
-
-        # Prevent killcursors from reaching the server while a getmore is in
-        # progress -- the server logs "Assertion: 16089:Cannot kill active
-        # cursor."
-        time.sleep(2)
-
-        def raises_cursor_not_found():
-            try:
-                next(cursor)
-                return False
-            except CursorNotFound:
-                return True
-
-        wait_until(raises_cursor_not_found, 'close cursor')
-
-    def test_kill_cursors_with_tuple(self):
-        if (client_context.is_mongos
-                and not client_context.version.at_least(2, 4, 7)):
-            # Old mongos sends incorrectly formatted error response when
-            # cursor isn't found, see SERVER-9738.
-            raise SkipTest("Can't test kill_cursors against old mongos")
-
-        self.collection = self.client.pymongo_test.test
-        self.collection.drop()
-
-        self.collection.insert_many([{'_id': i} for i in range(200)])
-        cursor = self.collection.find().batch_size(1)
-        next(cursor)
-        self.client.kill_cursors(
-            [cursor.cursor_id],
-            self.client.address)
-
-        # Prevent killcursors from reaching the server while a getmore is in
-        # progress -- the server logs "Assertion: 16089:Cannot kill active
-        # cursor."
-        time.sleep(2)
-
-        def raises_cursor_not_found():
-            try:
-                next(cursor)
-                return False
-            except CursorNotFound:
-                return True
-
-        wait_until(raises_cursor_not_found, 'close cursor')
-
-    def test_kill_cursors_with_server_unavailable(self):
-        with client_knobs(kill_cursor_frequency=9999999):
-            client = MongoClient('doesnt exist', connect=False,
-                                 serverSelectionTimeoutMS=0)
-
-            # Wait for the first tick of the periodic kill-cursors to pass.
-            time.sleep(1)
-
-            # Enqueue a kill-cursors message.
-            client.close_cursor(1234, ('doesnt-exist', 27017))
-
-            with warnings.catch_warnings(record=True) as user_warnings:
-                client._process_kill_cursors_queue()
-
-            self.assertIn("couldn't close cursor on ('doesnt-exist', 27017)",
-                          str(user_warnings[0].message))
-
     def test_lazy_connect_w0(self):
         # Ensure that connect-on-demand works when the first operation is
         # an unacknowledged write. This exercises _writable_max_wire_version().
@@ -859,8 +924,8 @@ class TestClient(IntegrationTest):
 
     @client_context.require_no_replica_set
     def test_connect_to_standalone_using_replica_set_name(self):
-        client = MongoClient(pair, replicaSet='anything',
-                             serverSelectionTimeoutMS=100)
+        client = single_client(replicaSet='anything',
+                               serverSelectionTimeoutMS=100)
 
         with self.assertRaises(AutoReconnect):
             client.test.test.find_one()
@@ -871,14 +936,60 @@ class TestClient(IntegrationTest):
         # the topology before the getMore message is sent. Test that
         # MongoClient._send_message_with_response handles the error.
         with self.assertRaises(AutoReconnect):
-            client = MongoClient(host, port, connect=False,
-                                 serverSelectionTimeoutMS=100,
-                                 replicaSet=client_context.replica_set_name)
+            client = rs_client(connect=False,
+                               serverSelectionTimeoutMS=100)
             client._send_message_with_response(
                 operation=message._GetMore('pymongo_test', 'collection',
                                            101, 1234, client.codec_options),
                 address=('not-a-member', 27017))
 
+    def test_heartbeat_frequency_ms(self):
+        class HeartbeatStartedListener(ServerHeartbeatListener):
+            def __init__(self):
+                self.results = []
+
+            def started(self, event):
+                self.results.append(event)
+
+            def succeeded(self, event):
+                pass
+
+            def failed(self, event):
+                pass
+
+        old_init = ServerHeartbeatStartedEvent.__init__
+
+        def init(self, *args):
+            old_init(self, *args)
+            self.time = time.time()
+
+        try:
+            ServerHeartbeatStartedEvent.__init__ = init
+            listener = HeartbeatStartedListener()
+            uri = "mongodb://%s:%d/?heartbeatFrequencyMS=500" % (
+                client_context.host, client_context.port)
+            client = single_client(uri, event_listeners=[listener])
+            wait_until(lambda: len(listener.results) >= 2,
+                       "record two ServerHeartbeatStartedEvents")
+
+            events = listener.results
+
+            # Default heartbeatFrequencyMS is 10 sec. Check the interval was
+            # closer to 0.5 sec with heartbeatFrequencyMS configured.
+            self.assertAlmostEqual(
+                events[1].time - events[0].time, 0.5, delta=2)
+
+            client.close()
+        finally:
+            ServerHeartbeatStartedEvent.__init__ = old_init
+
+    def test_small_heartbeat_frequency_ms(self):
+        uri = "mongodb://example/?heartbeatFrequencyMS=499"
+        with self.assertRaises(ConfigurationError) as context:
+            MongoClient(uri)
+
+        self.assertIn('heartbeatFrequencyMS', str(context.exception))
+
 
 class TestExhaustCursor(IntegrationTest):
     """Test that clients properly handle errors from exhaust cursors."""
