@@ -71,6 +71,9 @@ class MockMonitor(object):
     def close(self):
         pass
 
+    def remove_stale_sockets(self):
+        pass
+
 
 def create_mock_topology(uri, monitor_class=MockMonitor):
     # Some tests in the spec include URIs like mongodb://A/?connect=direct,
@@ -148,6 +151,14 @@ def check_outcome(self, topology, outcome):
             expected_server['setName'],
             actual_server_description.replica_set_name)
 
+        self.assertEqual(
+            expected_server.get('setVersion'),
+            actual_server_description.set_version)
+
+        self.assertEqual(
+            expected_server.get('electionId'),
+            actual_server_description.election_id)
+
     self.assertEqual(outcome['setName'], topology.description.replica_set_name)
     expected_topology_type = getattr(TOPOLOGY_TYPE, outcome['topologyType'])
     self.assertEqual(topology_type_name(expected_topology_type),
