@@ -35,8 +35,6 @@ from gridfs.errors import CorruptGridFile, FileExists, NoFile
 from test.test_replica_set_client import TestReplicaSetClientBase
 from test import (client_context,
                   unittest,
-                  host,
-                  port,
                   IntegrationTest)
 from test.utils import (joinall,
                         single_client,
@@ -81,8 +79,7 @@ class TestGridfsNoConnect(unittest.TestCase):
 
     @classmethod
     def setUpClass(cls):
-        client = MongoClient(host, port, connect=False)
-        cls.db = client.pymongo_test
+        cls.db = MongoClient(connect=False).pymongo_test
 
     def test_gridfs(self):
         self.assertRaises(TypeError, gridfs.GridFS, "foo")
@@ -462,6 +459,11 @@ class TestGridfs(IntegrationTest):
 
 class TestGridfsReplicaSet(TestReplicaSetClientBase):
 
+    @classmethod
+    @client_context.require_secondaries_count(1)
+    def setUpClass(cls):
+        super(TestGridfsReplicaSet, cls).setUpClass()
+
     def test_gridfs_replica_set(self):
         rsc = rs_client(
             w=self.w,
@@ -513,9 +515,8 @@ class TestGridfsReplicaSet(TestReplicaSetClientBase):
         self.assertRaises(ConnectionFailure, fs.put, 'data')
 
     def tearDown(self):
-        rsc = client_context.rs_client
-        rsc.pymongo_test.drop_collection('fs.files')
-        rsc.pymongo_test.drop_collection('fs.chunks')
+        self.client.pymongo_test.drop_collection('fs.files')
+        self.client.pymongo_test.drop_collection('fs.chunks')
 
 
 if __name__ == "__main__":
