@@ -22,7 +22,7 @@ sys.path[0:0] = [""]
 
 from bson.errors import InvalidId
 from bson.objectid import ObjectId
-from bson.py3compat import PY3, u, _unicode
+from bson.py3compat import PY3, _unicode
 from bson.tz_util import (FixedOffset,
                           utc)
 from test import SkipTest, unittest
@@ -51,13 +51,13 @@ class TestObjectId(unittest.TestCase):
         a = ObjectId()
         self.assertEqual(a, ObjectId(_unicode(a)))
         self.assertEqual(ObjectId("123456789012123456789012"),
-                         ObjectId(u("123456789012123456789012")))
-        self.assertRaises(InvalidId, ObjectId, u("hello"))
+                         ObjectId(u"123456789012123456789012"))
+        self.assertRaises(InvalidId, ObjectId, u"hello")
 
     def test_from_hex(self):
         ObjectId("123456789012123456789012")
         self.assertRaises(InvalidId, ObjectId, "123456789012123456789G12")
-        self.assertRaises(InvalidId, ObjectId, u("123456789012123456789G12"))
+        self.assertRaises(InvalidId, ObjectId, u"123456789012123456789G12")
 
     def test_repr_str(self):
         self.assertEqual(repr(ObjectId("1234567890abcdef12345678")),
@@ -123,11 +123,6 @@ class TestObjectId(unittest.TestCase):
             self.assertEqual(orig, pickle.loads(pkl))
 
     def test_pickle_backwards_compatability(self):
-        # For a full discussion see http://bugs.python.org/issue6137
-        if sys.version.startswith('3.0'):
-            raise SkipTest("Python 3.0.x can't unpickle "
-                           "objects pickled in Python 2.x.")
-
         # This string was generated by pickling an ObjectId in pymongo
         # version 1.9
         pickled_with_1_9 = (
