@@ -105,11 +105,6 @@ class TestSON(unittest.TestCase):
             self.assertEqual(pickled['son'], pickled['list'][1])
 
     def test_pickle_backwards_compatability(self):
-        # For a full discussion see http://bugs.python.org/issue6137
-        if sys.version.startswith('3.0'):
-            raise SkipTest("Python 3.0.x can't unpickle "
-                           "objects pickled in Python 2.x.")
-
         # This string was generated by pickling a SON object in pymongo
         # version 2.1.1
         pickled_with_2_1_1 = b(
